﻿// Beispiel Programm LogFile (C# Window Projekt Microsoft Visual Studio 2010)
//
// Beschreibung: Verbindet mit einer LStep Express Steuerung auf COM1 und COM9 (siehe ConnectController)
//               Anschließend kann ein relativer Move oder ein Fehler  (siehe SendError) ausgelöst werden
//               Der fehlercode des Befehls wird in einer ListBox ausgegeben. Weiterhin ist es möglich,
//               das Protokollfenster sowie den Fehlerkanal (siehe EnableErrorChannel) zu aktiveren. Bei 
//               aktivem Fehlerkanal werden asynchrone Fehler in einer ListBox angezeigt. Soll eine 
//               Log-Datei geschrieben werden, kann die Anzahl der Zeilen und der Dateiname festgelegt 
//               werden (siehe BtnStartLogFile_Click). Weiterhin kann die Funktionalität der 
//               CallBackFunktionen gestestet werden. (siehe SetExtCallBackFct, SetOsziCallBackFct, 
//               AddErrorChannelMessage, LSTEPExtCallBackFunction, LSTEPCallBackFunction)
//
// Erzeugt von: Andreas Köstler, 22.04.2014 , LANG GmbH & Co. KG

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices; // Marshal
using CClassLStep;

namespace CS_LogFile
{
    public partial class LogFile : Form
    {
        private ControllerObject[] Con = new ControllerObject[2]; // Variable für LSTEP Steuerungen

        private event ErrorChannelMessageEventHandler ErrorChannelMessageEvent; // Events für den ErrorChannel

        public LogFile()
        {
            InitializeComponent();
            ErrorChannelMessageEvent += new ErrorChannelMessageEventHandler(AddErrorChannelMessage); // Füge Funktion für Fehlerhandling hinzu
            Con[0] = new ControllerObject(); // Erzeuge Steuerungs-Elemente
            Con[1] = new ControllerObject();
        }

        // Funktion zum auslösen eines Events
        protected virtual void OnErrorChannelMessage(ErrorChannelMessageEventArgs e)
        {
            if (ErrorChannelMessageEvent != null)
            {
                ErrorChannelMessageEvent(this, e);
            }
        }

        // Standard Callbackfunktion z.B. für ErrorChannel Massages
        // Die Callbackfunktion sollte nur zur Signalisierung bzw. zum weiterreichen von Fehlermeldungen verwendet werden!
        // Aus dieser Callbackfunktion kann kein Rückschluss auf die Steuerung geschlossen werden, von der der Callback
        // ausgelöst wurde.
        //
        // Wurde einer Steuerung eine Standard Callbackfunktion sowie eine Extended Callbackfunktion zugewiesen, wird nur
        // die Extended Callbackfunktion ausgeführt.
        private void LSTEPCallBackFunction(IntPtr Data, int MaxLen, int ChannelID)
        {
            // Konvertiere die Ansi Zeichenkette zum Datentyp String
            string DataString = System.String.Empty;
            DataString = Marshal.PtrToStringAnsi(Data);

            //  Bedeutung der ChannelID der Callbackfunktion
            //  1 = Oszilloskop Daten
            //  2 = Fehler/Zusatzhinweise der Steuerung
            //  3 = Oszilloskop Aufzeichnungsinformationen
            //  4... = Noch nicht belegt

            if (ChannelID == 2)
            {
                // Erzeuge Event
                ErrorChannelMessageEventArgs e = new ErrorChannelMessageEventArgs(DataString);
                OnErrorChannelMessage(e);
            }
        }


        // Extended Callbackfunktion z.B. für ErrorChannel Massages
        // Die Callbackfunktion sollte nur zur Signalisierung bzw. zum weiterreichen von Fehlermeldungen verwendet werden!
        // Über das Objekt hinter IntPtr pObject kann die auslösende Steuerung identifiziert werden. Der Zeiger auf das Objekt
        // wird in der Funktion SetExtCallBackFct() als Parameter übergeben.
        //
        // Wurde einer Steuerung eine Standard Callbackfunktion sowie eine Extended Callbackfunktion zugewiesen, wird nur
        // die Extended Callbackfunktion ausgeführt.
        private int LSTEPExtCallBackFunction(IntPtr Data, int MaxLen, int ChannelID, IntPtr pObject)
        {
            // Konvertiere die Ansi Zeichenkette zum Datentyp String
            string DataString = System.String.Empty;
            DataString = Marshal.PtrToStringAnsi(Data);
            //  Bedeutung der ChannelID der Callbackfunktion
            //  1 = Oszilloskop Daten
            //  2 = Fehler/Zusatzhinweise der Steuerung
            //  3 = Oszilloskop Aufzeichnungsinformationen
            //  4... = Noch nicht belegt

            if (ChannelID == 2 && pObject != IntPtr.Zero)
            {
                // Erzeuge Event
                ErrorChannelMessageEventArgs e = new ErrorChannelMessageEventArgs(DataString, pObject);
                OnErrorChannelMessage(e);
            }
            return 0;
        }

        // Funktion zum Verarbeiten der Callback-Funktionsaufrufe
        private void AddErrorChannelMessage(object sender, ErrorChannelMessageEventArgs e)
        {
            string DataString = System.String.Empty;
            string DataStringOut = System.String.Empty;
            string TranslErrorString = System.String.Empty;
            int MessageType = 0, Achse = 0, LSID = -1;
            GCHandle LS_Handle;
            LStep SenderLStep;
            string[] DataParts;

            // Darf der Eintrag in die listBox durchgeführt werden?
            if (listBoxErrorChannelMessages.InvokeRequired)
            {
                // Bereite Event vor um im korrekten Thread eine Ausgabe zu generieren
                Invoke(ErrorChannelMessageEvent, new object[] { sender, e });
            }
            else
            {
                DataString = e.GetErrorChannelMessage(); // Kopiere Fehlermeldung nach DataString
                if (e.GetSenderObject() != IntPtr.Zero)
                {
                    LS_Handle = (GCHandle)e.GetSenderObject();
                    SenderLStep = (LS_Handle.Target as LStep);
                    LSID = SenderLStep.GetLSID(); // Hole LSID
                    DataStringOut = "Control Nr: " + LSID.ToString();
                    listBoxErrorChannelMessages.Items.Add(DataStringOut); // Füge LSID in listBox ein
                    DataStringOut = System.String.Empty;
                    SenderLStep.TranslateErrMsg(DataString, DataStringOut, 1000); // Übersetze Datastring in eine lesbare Meldung
                }
                else
                {
                    Con[0].LS.TranslateErrMsg(DataString, DataStringOut, 1000); // Übersetze Datastring in eine lesbare Meldung
                }

                listBoxErrorChannelMessages.Items.Add(DataStringOut); // Füge Meldung in listBox ein

                DataParts = DataString.Split(new Char[] { '\t' }); // Splitte DataString auf in Teilstrings. Trenner für Zerlegung ist Tabulator
                MessageType = Convert.ToInt32(DataParts[0]); // Konvertiere 1. Teilstring in Nachrichtentyp

                //Ausschlüsselung der Nachrichtentypen in DataParts[0]
                //  0 = in der Regel einem Fehler
                //  99 = Hinweis zu einem Fehlern
                switch (MessageType) // Führe Nachrichtenabhängigen Code aus
                {
                    case 0: // Fehler
                        
                        // Aufschlüsselung von DataParts[1] bis DataParts[4]
                        //  1.	Die Achsnummer. 0 entspricht der X-Achse … 5 entspricht einer nicht Achsgebundenen Meldung
                        //  2.	Fehlernummer. Die Fehlerliste kann in der Dokumentation nachgelesen werden.
                        //  3.	Zeit in Sekunden seit dem Start der Steuerung.
                        //  4.	Zeit in Millisekunden als Bruchteil der Betriebssekunden.

                        Achse = Convert.ToInt32(DataParts[1]); // Wandle Achsnummer in Zahl
                        if (Achse > 4) // ist Nummer größer 4 ist es keine achsspezifische Fehlernummer
                        {
                            DataParts[1] = "-";
                        }

                        // Kodiere eigene Fehlermeldung
                        DataStringOut = "Achse " + DataParts[1] + "; Fehler Nr " + DataParts[2] + "; Betriebssekunden " + DataParts[3] + "s " + DataParts[4].Trim() + "ms";
                        listBoxErrorChannelMessages.Items.Add(DataStringOut); // Füge Meldung in listBox ein
                        break;
                    case 99: // Hinweis

                        // Aufschlüsselung von DataParts[1]
                        // 1. String als Klartext
                        DataStringOut = "Hinweis: " + DataParts[1];
                        listBoxErrorChannelMessages.Items.Add(DataStringOut); // Füge Meldung in listBox ein
                        break;
                    default:
                        break;
                }
            }
        }

        private void BtnConnect_Click(object sender, EventArgs e)
        {
            ConnectController("com1", "DEU", ref Con[0], true);
        }

        private void BtnConnect2_Click(object sender, EventArgs e)
        {
            ConnectController("com9", "ENG", ref Con[1], true);
        }

        private void ConnectController(string ComName, string Language, ref ControllerObject rController, bool ExtCallBack)
        {
            GCHandle LS_Handle = GCHandle.Alloc(rController.LS);
            IntPtr LS_IntPtr = (IntPtr)LS_Handle;
            int Result = 0;
            if (ExtCallBack)
            {
                // Setze Extendet CallbackFunktion
                rController.LS.SetExtCallBackFct(LSTEPExtCallBackFunction, LS_IntPtr);
            }
            else
            {
                // Setze Standard CallbackFunktion
                rController.LS.SetOsziCallBackFct(LSTEPCallBackFunction);
            }

            rController.LS.SetLanguageW(Language);  // Setze Sprache

            Result = rController.LS.ConnectSimpleW(5, ComName, 115200, CheckBoxProtocolWindow.Checked);  // Baue eine Verbindung zur Steuerung auf
            if (Result == 0) rController.Connected = true;
            if (rController.Connected)
            {
                EnableErrorChannel(ref rController);
            }
        }

        private void BtnDisconnect_Click(object sender, EventArgs e)
        {
            Con[0].Connected = false; // Setze Merker auf nicht verbunden
            Con[0].LS.Disconnect(); // Trenne Verbindung zur Steuerung
        }

        private void BtnDisconnect2_Click(object sender, EventArgs e)
        {
            Con[1].Connected = false; // Setze Merker auf nicht verbunden
            Con[1].LS.Disconnect(); // Trenne Verbindung zur Steuerung
        }

        private void BtnMoveRel_Click(object sender, EventArgs e)
        {
            int ErrorNumber = 0;
            Con[0].LS.MoveRel(1.0, 1.0, 1.0, 1.0, true); // Führe relativen Move aus
            Con[0].LS.GetError(ref ErrorNumber);  // Rufe Fehler ab
            ListBoxErrorNumbers.Items.Add(ErrorNumber.ToString()); // Füge Fehler in listBox ein
        }
        private void BtnSendError_Click(object sender, EventArgs e)
        {
            SendError(ref Con[0]);
        }

        private void BtnSendError2_Click(object sender, EventArgs e)
        {
            SendError(ref Con[1]);
        }

        private void SendError(ref ControllerObject rController)
        {
            int ErrorNumber = 0;
            rController.LS.SendStringW("!FEHLER\r", null, 0, false, 100); // Da die Steuerung diesen Befehl nicht kennt, wir ein Fehler erzeugt
            rController.LS.GetError(ref ErrorNumber);  // Rufe Fehlercode ab
            ListBoxErrorNumbers.Items.Add(ErrorNumber.ToString()); // Füge Fehler in listBox ein
        }

        private void BtnStartLogFile_Click(object sender, EventArgs e)
        {
            TextBoxLogFileName.Enabled = false;
            Con[0].LS.SetExtValue(4, (int) numericUpDownLineCount.Value); // Setze Zeilenlänge, ab deren Überschreitung eine neue Textdatei erzeugt wird
            Con[0].LS.SetWriteLogTextFNW(false, null); // Deaktiviere Logging
            Con[0].LS.SetWriteLogTextFNW(true, "./" + TextBoxLogFileName.Text); // Aktiviere loggen in Textdatei
            TextBoxLogFileName.Enabled = true;
        }

        private void BtnFreeLogFile_Click(object sender, EventArgs e)
        {
            Con[0].LS.SetWriteLogTextFNW(false, null); // Deaktiviere Logging
        }

        private void checkBoxErrorChannel_CheckedChanged(object sender, EventArgs e)
        {
            EnableErrorChannel(ref Con[0]);
            EnableErrorChannel(ref Con[1]);
        }

        private void EnableErrorChannel(ref ControllerObject rController)
        {
            if (rController.Connected == true)
            {
                if (checkBoxErrorChannel.Checked == true)
                {
                    rController.LS.SendStringW("!errorchannel 2\r", null, 0, false, 100); // Aktiviere ErrorChannel
                }
                else
                {
                    rController.LS.SendStringW("!errorchannel 0\r", null, 0, false, 100); // Deaktiviere ErrorChannel
                }
            }
        }

        private void buttonClearError_Click(object sender, EventArgs e)
        {
            listBoxErrorChannelMessages.Items.Clear();
            ListBoxErrorNumbers.Items.Clear();
        }

        private void CheckBoxProtocolWindow_CheckedChanged(object sender, EventArgs e)
        {
            Con[0].LS.SetShowProt(CheckBoxProtocolWindow.Checked);
            Con[1].LS.SetShowProt(CheckBoxProtocolWindow.Checked);
        }
    }

    // Deklaration des EvendHanlders für die ErrorChannelMessages
    public delegate void ErrorChannelMessageEventHandler(object sender, ErrorChannelMessageEventArgs e);

    public class ErrorChannelMessageEventArgs : EventArgs
    {
        private string ErrorChannelMessage;
        private IntPtr SenderObject;

        public ErrorChannelMessageEventArgs(string ToSet, IntPtr pObject)
        {
            ErrorChannelMessage = ToSet;
            SenderObject = pObject;
        }

        public ErrorChannelMessageEventArgs(string ToSet)
        {
            ErrorChannelMessage = ToSet;
            SenderObject = IntPtr.Zero;
        }

        public string GetErrorChannelMessage()
        {
            return ErrorChannelMessage;
        }

        public IntPtr GetSenderObject()
        {
            return SenderObject;
        }
    }

    // Objekt zur Steuerungskontrolle
    public partial class ControllerObject : Object
    {
        public LStep LS; // Steuerungs-Objekt
        public bool Connected; // Indikator für Verbindungsstatus

        public ControllerObject()
        {
            LS = new LStep();
            Connected = false;
        }
    }
}
